@@ -4,7 +4,6 @@ from tests import unittest
 
 from botocore import model
 from botocore.compat import OrderedDict
-from botocore.exceptions import MissingServiceIdError
 
 
 @pytest.mark.parametrize("property_name", ['api_version', 'protocol'])
@@ -164,6 +163,36 @@ class TestOperationModelFromService(unittest.TestCase):
                     },
                     'errors': [{'shape': 'NoSuchResourceException'}],
                     'documentation': 'Docs for OperationTwo',
+                },
+                'PayloadOperation': {
+                    'http': {
+                        'method': 'POST',
+                        'requestUri': '/',
+                    },
+                    'name': 'PayloadOperation',
+                    'input': {
+                        'shape': 'PayloadOperationRequest'
+                    },
+                    'output': {
+                        'shape': 'PayloadOperationResponse',
+                    },
+                    'errors': [{'shape': 'NoSuchResourceException'}],
+                    'documentation': 'Docs for PayloadOperation',
+                },
+                'NoBodyOperation': {
+                    'http': {
+                        'method': 'GET',
+                        'requestUri': '/',
+                    },
+                    'name': 'NoBodyOperation',
+                    'input': {
+                        'shape': 'NoBodyOperationRequest'
+                    },
+                    'output': {
+                        'shape': 'OperationNameResponse',
+                    },
+                    'errors': [{'shape': 'NoSuchResourceException'}],
+                    'documentation': 'Docs for NoBodyOperation',
                 }
             },
             'shapes': {
@@ -182,13 +211,48 @@ class TestOperationModelFromService(unittest.TestCase):
                         }
                     }
                 },
+                'PayloadOperationRequest': {
+                    'type': 'structure',
+                    'members': {
+                        'Arg1': {'shape': 'TestConfig'},
+                        'Arg2': {'shape': 'stringType'},
+                    },
+                    'payload': 'Arg1'
+                },
+                'PayloadOperationResponse': {
+                    'type': 'structure',
+                    'members': {
+                        'String': {
+                            'shape': 'stringType',
+                        }
+                    },
+                    'payload': 'String'
+                },
+                'NoBodyOperationRequest': {
+                    'type': 'structure',
+                    'members': {
+                        'data': {
+                            'location': 'header',
+                            'locationName': 'x-amz-data',
+                            'shape': 'stringType'
+                        }
+                    }
+                },
                 'NoSuchResourceException': {
                     'type': 'structure',
                     'members': {}
                 },
                 'stringType': {
                     'type': 'string',
-                }
+                },
+                'TestConfig': {
+                    'type': 'structure',
+                    'members': {
+                        'timeout': {
+                            'shape': 'stringType'
+                        }
+                    }
+                },
             }
         }
         self.service_model = model.ServiceModel(self.model)
@@ -342,6 +406,38 @@ class TestOperationModelFromService(unittest.TestCase):
         operation_name = self.service_model.operation_model('OperationName')
         self.assertIsNone(operation_name.endpoint_discovery)
 
+    def test_http_checksum_absent(self):
+        operation_name = self.service_model.operation_model('OperationName')
+        self.assertEqual(operation_name.http_checksum, {})
+
+    def test_http_checksum_present(self):
+        operation = self.model['operations']['OperationName']
+        operation['httpChecksum'] = {
+            "requestChecksumRequired": True,
+            "requestAlgorithmMember": "ChecksumAlgorithm",
+            "requestValidationModeMember": "ChecksumMode",
+            "responseAlgorithms": ["crc32", "crc32c", "sha256", "sha1"],
+        }
+        service_model = model.ServiceModel(self.model)
+        operation_model = service_model.operation_model('OperationName')
+        http_checksum = operation_model.http_checksum
+        self.assertEqual(
+            http_checksum["requestChecksumRequired"],
+            True,
+        )
+        self.assertEqual(
+            http_checksum["requestAlgorithmMember"],
+            "ChecksumAlgorithm",
+        )
+        self.assertEqual(
+            http_checksum["requestValidationModeMember"],
+            "ChecksumMode",
+        )
+        self.assertEqual(
+            http_checksum["responseAlgorithms"],
+            ["crc32", "crc32c", "sha256", "sha1"],
+        )
+
 
 class TestOperationModelEventStreamTypes(unittest.TestCase):
     def setUp(self):
@@ -573,11 +669,15 @@ class TestDeepMerge(unittest.TestCase):
         # map_merged has a serialization as a member trait as well as
         # in the StrToStrMap.
         # The member trait should have precedence.
-        self.assertEqual(map_merged.serialization,
-                         # member beats the definition.
-                         {'name': 'Attribute',
-                          # From the definition.
-                          'flattened': True,})
+        self.assertEqual(
+            map_merged.serialization,
+            # member beats the definition.
+            {
+                'name': 'Attribute',
+                # From the definition.
+                'flattened': True,
+            }
+        )
         # Ensure we don't merge/mutate the original dicts.
         self.assertEqual(map_merged.key.serialization['name'], 'Name')
         self.assertEqual(map_merged.value.serialization['name'], 'Value')
@@ -696,7 +796,7 @@ class TestShapeResolver(unittest.TestCase):
                 }
             },
             'passwordType': {
-                "type":"string",
+                "type": "string",
             }
         }
         resolver = model.ShapeResolver(shapes)
@@ -740,10 +840,10 @@ class TestShapeResolver(unittest.TestCase):
                 }
             },
             'passwordType': {
-                "type":"string",
-                "min":1,
-                "max":128,
-                "sensitive":True
+                "type": "string",
+                "min": 1,
+                "max": 128,
+                "sensitive": True
             }
         }
         resolver = model.ShapeResolver(shapes)
@@ -778,7 +878,7 @@ class TestShapeResolver(unittest.TestCase):
     def test_shape_list(self):
         shapes = {
             'mfaDeviceListType': {
-                "type":"list",
+                "type": "list",
                 "member": {"shape": "MFADevice"},
             },
             'MFADevice': {
